﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;

namespace PngEncoder
{
    class PngEncoderDemo
    {
        // Creates a simple PNG image via the PngEncoder class.
        static void Main()
        {
            //using (var stream = Stream.Null)
            using (var stream = File.Create("test.png"))
            {
                var width = 500;
                var height = 300;

                //var rand = new Random(1);
                //var pixels = Enumerable.Range(0, width * height).Select(i => rand.Next(int.MinValue, int.MaxValue));
                var pixels = new TrackingEnumerable<int>(width * height, (int)0x00ff00 | (0xff << 24));

                var contents = Delay.PngEncoder.Encode(width, height,  pixels);
                foreach (var b in contents)
                {
                    stream.WriteByte(b);
                }
            }
        }
    }

    // Class that returns a TrackingEnumerator to help detect multiple calls to GetEnumerator.
    class TrackingEnumerable<T> : IEnumerable<T>
    {
        private readonly int _count;
        private readonly T _value;

        public TrackingEnumerable(int count, T value)
        {
            _count = count;
            _value = value;
        }

        public IEnumerator<T> GetEnumerator()
        {
            return new TrackingEnumerator<T>(_count, _value);
        }

        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }
    }

    // Class that helps detect multiple enumerator instances by outputting from its constructor.
    class TrackingEnumerator<T> : IEnumerator<T>
    {
        private int _count;
        private readonly T _value;

        public TrackingEnumerator(int count, T value)
        {
            Debug.WriteLine("new TrackingEnumerator");
            _count = count + 1;
            _value = value;
        }

        public T Current
        {
            get { return _value; }
        }

        object IEnumerator.Current
        {
            get { return Current; }
        }

        public bool MoveNext()
        {
            _count--;
            return (0 < _count);
        }

        public void Reset() { }

        public void Dispose() { }
    }
}
