﻿using System;
using System.IO;
using System.Linq;
using Delay;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace PngEncoderTests
{
    [TestClass]
    public class PngEncoderTests
    {
        [TestMethod]
        public void RandomImages()
        {
            var rand = new Random(1);
            for (var i = 0; i < 25; i++)
            {
                var width = rand.Next(1, 500);
                var height = rand.Next(1, 500);
                var pixels = new int[width * height];
                for (var p = 0; p < pixels.Length; p++)
                {
                    pixels[p] = rand.Next(int.MinValue, int.MaxValue);
                }
                var pngBytes = PngEncoder.Encode(width, height, pixels).ToArray();

                // Verify with System.Drawing.Bitmap
                using (var stream = new MemoryStream(pngBytes))
                {
                    var bitmap = new System.Drawing.Bitmap(stream);
                    Assert.AreEqual(width, bitmap.Width);
                    Assert.AreEqual(height, bitmap.Height);
                    for (var y = 0; y < height; y++)
                    {
                        for (var x = 0; x < width; x++)
                        {
                            var pixel = pixels[(y * width) + x];
                            var color = bitmap.GetPixel(x, y);
                            Assert.AreEqual((byte)(pixel >> 24), color.A);
                            Assert.AreEqual((byte)(pixel >> 16), color.R);
                            Assert.AreEqual((byte)(pixel >> 8), color.G);
                            Assert.AreEqual((byte)(pixel >> 0), color.B);
                        }
                    }
                }

                // Verify with System.Windows.Media.Imaging.PngBitmapDecoder
                using (var stream = new MemoryStream(pngBytes))
                {
                    var bitmap = new System.Windows.Media.Imaging.PngBitmapDecoder(stream, System.Windows.Media.Imaging.BitmapCreateOptions.None, System.Windows.Media.Imaging.BitmapCacheOption.None);
                    Assert.AreEqual(1, bitmap.Frames.Count);
                    var frame = bitmap.Frames[0];
                    Assert.AreEqual(width, frame.PixelWidth);
                    Assert.AreEqual(height, frame.PixelHeight);
                    var framePixels = new int[width * height];
                    frame.CopyPixels(framePixels, width * 4, 0);
                    Assert.IsTrue(Enumerable.SequenceEqual(pixels, framePixels));
                }
            }
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void ZeroWidth()
        {
            PngEncoder.Encode(0, 10, new int[1]);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void ZeroHeight()
        {
            PngEncoder.Encode(10, 0, new int[1]);
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NullPixels()
        {
            PngEncoder.Encode(10, 10, null);
        }

        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void NotEnoughPixels()
        {
            PngEncoder.Encode(10, 10, new int[99]).ToArray();
        }

        [TestMethod]
        public void DeferredExecution()
        {
            PngEncoder.Encode(10, 10, new int[0]);
        }

        [TestMethod]
        public void TooManyPixels()
        {
            PngEncoder.Encode(10, 10, new int[101]);
        }
    }
}
